@@ -78,7 +78,7 @@ public class JavacTaskImpl extends JavacTask {
     private TaskListener taskListener;
     private AtomicBoolean used = new AtomicBoolean();
     private Iterable<? extends Processor> processors;
-
+    
     private Integer result = null;
 
     JavacTaskImpl(JavacTool tool,
@@ -139,6 +139,7 @@ public class JavacTaskImpl extends JavacTask {
     public Boolean call() {
         if (!used.getAndSet(true)) {
             beginContext();
+            notYetEntered = new HashMap<JavaFileObject, JCCompilationUnit>();
             try {
                 compilerMain.setFatalErrors(true);
                 result = compilerMain.compile(args, context, fileObjects, processors);
@@ -149,6 +150,7 @@ public class JavacTaskImpl extends JavacTask {
             args = null;
             context = null;
             fileObjects = null;
+            notYetEntered = null;
             return result == 0;
         } else {
             throw new IllegalStateException("multiple calls to method 'call'");
@@ -163,6 +165,10 @@ public class JavacTaskImpl extends JavacTask {
         this.processors = processors;
     }
 
+    public Iterable<? extends Processor> getProcessors() {
+        return processors;
+    }
+
     public void setLocale(Locale locale) {
         if (used.get())
             throw new IllegalStateException();
@@ -376,9 +382,11 @@ public class JavacTaskImpl extends JavacTask {
         try {
             List<JCCompilationUnit> units = compiler.enterTrees(roots.toList());
 
-            if (notYetEntered.isEmpty())
+            if (!compiler.skipAnnotationProcessing)
                 compiler = compiler.processAnnotations(units);
 
+            compiler.flushTempDiags();
+
             ListBuffer<TypeElement> elements = new ListBuffer<TypeElement>();
             for (JCCompilationUnit unit : units) {
                 boolean isPkgInfo = unit.sourcefile.isNameCompatible("package-info",
@@ -388,7 +396,7 @@ public class JavacTaskImpl extends JavacTask {
                         elements.append(unit.packge.package_info);
                 } else {
                     for (JCTree node : unit.defs)
-                        if (node.getTag() == JCTree.CLASSDEF)
+                        if (node.getTag() == JCTree.CLASSDEF && ((JCTree.JCClassDecl) node).sym != null)
                             elements.append(((JCTree.JCClassDecl) node).sym);
                 }
             }
@@ -473,6 +481,9 @@ public class JavacTaskImpl extends JavacTask {
                 };
                 f.run(compiler.todo, classes);
             }
+            if (!compiler.skipAnnotationProcessing)
+                compiler = compiler.processAnnotations(List.<JCCompilationUnit>nil());
+            compiler.flushTempDiags();
         } finally {
             compiler.log.flush();
         }
@@ -652,6 +663,8 @@ public class JavacTaskImpl extends JavacTask {
             throw new IllegalArgumentException();
         compiler = JavaCompiler.instance(context);
         JavaFileObject prev = compiler.log.useSource(null);
+        boolean old = compiler.log.suppressErrorsAndWarnings;
+        compiler.log.suppressErrorsAndWarnings = true;
         ParserFactory parserFactory = ParserFactory.instance(context);
         Attr attr = Attr.instance(context);
         try {
@@ -660,6 +673,7 @@ public class JavacTaskImpl extends JavacTask {
             JCTree tree = parser.parseType();
             return attr.attribType(tree, (Symbol.TypeSymbol)scope);
         } finally {
+            compiler.log.suppressErrorsAndWarnings = old;
             compiler.log.useSource(prev);
         }
     }
@@ -669,6 +683,8 @@ public class JavacTaskImpl extends JavacTask {
             throw new IllegalArgumentException();
         compiler = JavaCompiler.instance(context);
         JavaFileObject prev = compiler.log.useSource(null);
+        boolean old = compiler.log.suppressErrorsAndWarnings;
+        compiler.log.suppressErrorsAndWarnings = true;
         ParserFactory parserFactory = ParserFactory.instance(context);
         try {
             CharBuffer buf = CharBuffer.wrap((stmt+"\u0000").toCharArray(), 0, stmt.length());
@@ -680,6 +696,7 @@ public class JavacTaskImpl extends JavacTask {
             }
             return null;
         } finally {
+            compiler.log.suppressErrorsAndWarnings = old;
             compiler.log.useSource(prev);
         }
     }
@@ -689,6 +706,8 @@ public class JavacTaskImpl extends JavacTask {
             throw new IllegalArgumentException();
             compiler = JavaCompiler.instance(context);
         JavaFileObject prev = compiler.log.useSource(null);
+        boolean old = compiler.log.suppressErrorsAndWarnings;
+        compiler.log.suppressErrorsAndWarnings = true;
         ParserFactory parserFactory = ParserFactory.instance(context);
         try {
             CharBuffer buf = CharBuffer.wrap((expr+"\u0000").toCharArray(), 0, expr.length());
@@ -700,6 +719,7 @@ public class JavacTaskImpl extends JavacTask {
             }
             return null;
         } finally {
+            compiler.log.suppressErrorsAndWarnings = old;
             compiler.log.useSource(prev);
         }
     }
@@ -709,6 +729,8 @@ public class JavacTaskImpl extends JavacTask {
             throw new IllegalArgumentException();
             compiler = JavaCompiler.instance(context);
         JavaFileObject prev = compiler.log.useSource(null);
+        boolean old = compiler.log.suppressErrorsAndWarnings;
+        compiler.log.suppressErrorsAndWarnings = true;
         ParserFactory parserFactory = ParserFactory.instance(context);
         try {
             CharBuffer buf = CharBuffer.wrap((init+"\u0000").toCharArray(), 0, init.length());
@@ -720,6 +742,7 @@ public class JavacTaskImpl extends JavacTask {
             }
             return null;
         } finally {
+            compiler.log.suppressErrorsAndWarnings = old;
             compiler.log.useSource(prev);
         }
     }
@@ -729,6 +752,8 @@ public class JavacTaskImpl extends JavacTask {
             throw new IllegalArgumentException();
             compiler = JavaCompiler.instance(context);
         JavaFileObject prev = compiler.log.useSource(null);
+        boolean old = compiler.log.suppressErrorsAndWarnings;
+        compiler.log.suppressErrorsAndWarnings = true;
         ParserFactory parserFactory = ParserFactory.instance(context);
         try {
             CharBuffer buf = CharBuffer.wrap((block+"\u0000").toCharArray(), 0, block.length());
@@ -741,6 +766,7 @@ public class JavacTaskImpl extends JavacTask {
             }
             return null;
         } finally {
+            compiler.log.suppressErrorsAndWarnings = old;
             compiler.log.useSource(prev);
         }
     }
@@ -749,11 +775,14 @@ public class JavacTaskImpl extends JavacTask {
         Log log = Log.instance(context);
         Attr attr = Attr.instance(context);
         JavaFileObject prev = log.useSource(null);
+        boolean old = log.suppressErrorsAndWarnings;
+        log.suppressErrorsAndWarnings = true;
         try {
             if (tree instanceof JCExpression)
                 return attr.attribExpr(tree, env, Type.noType);
             return attr.attribStat(tree, env);
         } finally {
+            log.suppressErrorsAndWarnings = old;
             log.useSource(prev);
         }
     }
@@ -762,10 +791,13 @@ public class JavacTaskImpl extends JavacTask {
         Log log = Log.instance(context);
         Attr attr = Attr.instance(context);
         JavaFileObject prev = log.useSource(null);
+        boolean old = log.suppressErrorsAndWarnings;
+        log.suppressErrorsAndWarnings = true;
         try {
             Env<AttrContext> ret = tree instanceof JCExpression ? attr.attribExprToTree(tree, env, to) : attr.attribStatToTree(tree, env, to);
             return new JavacScope(ret);
         } finally {
+            log.suppressErrorsAndWarnings = old;
             log.useSource(prev);
         }
     }
@@ -791,8 +823,9 @@ public class JavacTaskImpl extends JavacTask {
             final Map<? super JCTree,? super String> docComments) {
         ParserFactory parserFactory = ParserFactory.instance(context);
         CharBuffer buf = CharBuffer.wrap((newBodyText+"\u0000").toCharArray(), 0, newBodyText.length());
-        JavacParser parser = (JavacParser) parserFactory.newParser(buf, ((JCBlock)methodToReparse.getBody()).pos, annonIndex,  ((JCCompilationUnit)topLevel).endPositions);
+        JavacParser parser = (JavacParser) parserFactory.newParser(buf, ((JCBlock)methodToReparse.getBody()).pos, ((JCCompilationUnit)topLevel).endPositions);
         final JCStatement statement = parser.parseStatement();
+        JavacParser.assignAnonymousClassIndices(Names.instance(context), statement, Names.instance(context).empty, annonIndex);
         if (statement.getKind() == Tree.Kind.BLOCK) {
             if (docComments != null) {
                 docComments.putAll(parser.getDocComments());
