@@ -27,6 +27,9 @@ package com.sun.tools.javac.comp;
 
 import java.net.URI;
 import java.util.*;
+import javax.lang.model.element.ExecutableElement;
+import javax.lang.model.element.TypeElement;
+import javax.lang.model.element.VariableElement;
 import javax.tools.JavaFileObject;
 import javax.tools.JavaFileManager;
 
@@ -41,6 +44,7 @@ import com.sun.tools.javac.util.List;
 import com.sun.tools.javac.code.Type.*;
 import com.sun.tools.javac.code.Symbol.*;
 import com.sun.tools.javac.tree.JCTree.*;
+import javax.lang.model.util.ElementScanner6;
 
 import static com.sun.tools.javac.code.Flags.*;
 import static com.sun.tools.javac.code.Kinds.*;
@@ -402,18 +406,23 @@ public class Enter extends JCTree.Visitor {
                         break;
                     }
                 }
-                if (c == null) {
+                if (c != null) {
+                    if (chk.compiled.get(c.flatname) != null) {
+                        c = null;
+                    } else {
+                        reattr = true;
+                        if (owner.kind == TYP) {
+                            if ((owner.flags_field & INTERFACE) != 0) {
+                                tree.mods.flags |= PUBLIC | STATIC;
+                            }
+                        }
+                        doEnterClass = false;
+                    }
+                } else if ((enclScope.owner.flags_field & APT_CLEANED) == 0) {
                     ClassSymbol cs = enclScope.owner.outermostClass();
                     treeLoader.couplingError(cs, tree);
-                } else {
-                    reattr = true;
-                    if (owner.kind == TYP) {
-                        if ((owner.flags_field & INTERFACE) != 0) {
-                            tree.mods.flags |= PUBLIC | STATIC;
-                        }
-                    }
+                    doEnterClass = false;
                 }
-                doEnterClass = false;
             }
             if (c == null) {
                 if (!tree.name.isEmpty() &&
@@ -469,19 +478,36 @@ public class Enter extends JCTree.Visitor {
             c.flags_field &= ~FROMCLASS;
             c.kind = TYP;
             c.type = new ClassType(Type.noType, List.<Type>nil(), c);
-        } else if (reattr) {
-            c.flags_field |= FROMCLASS;
+        } else if (reattr && c.completer == null) {
+            new ElementScanner6<Void, Void>() {
+                @Override
+                public Void visitType(TypeElement e, Void p) {
+                    if (e instanceof ClassSymbol)
+                        ((ClassSymbol) e).flags_field |= FROMCLASS;
+                    return super.visitType(e, p);
+                }
+                @Override
+                public Void visitExecutable(ExecutableElement e, Void p) {
+                    if (e instanceof MethodSymbol)
+                        ((MethodSymbol) e).flags_field |= FROMCLASS;
+                    return null;
+                }
+                @Override
+                public Void visitVariable(VariableElement e, Void p) {
+                    if (e instanceof VarSymbol)
+                        ((VarSymbol) e).flags_field |= FROMCLASS;
+                    return null;
+                }
+            }.scan(c);
         }
 
         // Enter class into `compiled' table and enclosing scope.
-        if (!reattr && !noctx) {
-            if (chk.compiled.get(c.flatname) != null) {
-                duplicateClass(tree.pos(), c);
-                result = types.createErrorType(tree.name, owner, Type.noType);
-                tree.sym = c = (ClassSymbol)result.tsym;
-            } else {
-                chk.compiled.put(c.flatname, c);
-            }
+        if (!reattr && !noctx && chk.compiled.get(c.flatname) != null) {
+            duplicateClass(tree.pos(), c);
+            result = types.createErrorType(tree.name, owner, Type.noType);
+            tree.sym = c = (ClassSymbol)result.tsym;
+        } else {
+            chk.compiled.put(c.flatname, c);
         }
         if (doEnterClass) {
             enclScope.enter(c);
@@ -521,7 +547,7 @@ public class Enter extends JCTree.Visitor {
             // Enter type parameters.
             ct.typarams_field = classEnter(tree.typarams, localEnv);
         } else {
-            c.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, c, tree) | (c.flags_field & FROMCLASS);
+            c.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, c, tree) | (c.flags_field & (FROMCLASS | APT_CLEANED));
             ClassType ct = (ClassType)c.type;
             boolean wasNull = false;
             if (ct.typarams_field != null) {
@@ -594,8 +620,10 @@ public class Enter extends JCTree.Visitor {
             }
             if (result != null)
                 return;
-            ClassSymbol cs = env.info.scope.owner.outermostClass();
-            treeLoader.couplingError(cs, tree);
+            if ((env.info.scope.owner.flags_field & APT_CLEANED) == 0) {
+                ClassSymbol cs = env.info.scope.owner.outermostClass();
+                treeLoader.couplingError(cs, tree);
+            }
         }
         TypeVar a = (tree.type != null)
         ? (TypeVar)tree.type
