@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2008 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright 1999-2009 Sun Microsystems, Inc.  All Rights Reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -35,7 +35,9 @@ import com.sun.tools.javac.util.JCDiagnostic.DiagnosticPosition;
 import com.sun.tools.javac.code.Symbol.*;
 import com.sun.tools.javac.tree.JCTree.*;
 
+import java.util.logging.Level;
 import java.util.logging.Logger;
+import javax.tools.JavaFileObject;
 import static com.sun.tools.javac.code.Flags.*;
 import static com.sun.tools.javac.code.Kinds.*;
 import static com.sun.tools.javac.code.TypeTags.*;
@@ -606,8 +608,9 @@ public class Flow extends TreeScanner {
                 boolean firstConstructor = true;
                 for (List<JCTree> l = tree.defs; l.nonEmpty(); l = l.tail) {
                     if (TreeInfo.isInitialConstructor(l.head)) {
-                        List<Type> mthrown =
-                            ((JCMethodDecl) l.head).sym.type.getThrownTypes();
+                        MethodSymbol sym = ((JCMethodDecl) l.head).sym;
+                        List<Type> mthrown = sym != null
+                                ? sym.type.getThrownTypes() : List.<Type>nil();
                         if (firstConstructor) {
                             caught = mthrown;
                             firstConstructor = false;
@@ -676,7 +679,22 @@ public class Flow extends TreeScanner {
     public void visitMethodDef(JCMethodDecl tree) {
         if (tree.body == null) return;
         if (tree.sym == null) {
-            Logger.getLogger(Flow.class.getName()).warning("Flow.visitMethodDef has a null tree.sym. Tree: [" + tree + "] Enclosing class: [" + classDef + "]"); //NOI18N
+            String methodName = String.valueOf(tree.name);
+            int methodStart = tree.getStartPosition();
+
+            if (classDef != null && classDef.sym != null) {
+                JavaFileObject file = classDef.sym.sourcefile != null ? classDef.sym.sourcefile : classDef.sym.classfile;
+                String source = file != null ? file.toUri().toASCIIString() : "<unknown>";
+                int classStart = classDef.getStartPosition();
+                String className = String.valueOf(classDef.sym.flatname);
+
+                Logger.getLogger(Flow.class.getName()).log(Level.WARNING, "Flow.visitMethodDef has a null tree.sym. Method name: {0}, offset: {1}, enclosing class name: {2}, offset: {3}, source file: {4}", new Object[] {methodName, methodStart, className, classStart, source}); //NOI18N
+            } else {
+                String className = classDef != null ? String.valueOf(classDef.name) : "<unknown>";
+
+                Logger.getLogger(Flow.class.getName()).log(Level.WARNING, "Flow.visitMethodDef has a null tree.sym. Method name: {0}, offset: {1}, enclosing class name: {2}", new Object[] {methodName, methodStart, className}); //NOI18N
+            }
+            Logger.getLogger(Flow.class.getName()).log(Level.FINER, "Flow.visitMethodDef has a null tree.sym. Tree: [{0}] Enclosing class: [{1}]", new Object[] {tree, classDef}); //NOI18N
             return;
         }
         List<Type> caughtPrev = caught;
@@ -749,7 +767,22 @@ public class Flow extends TreeScanner {
 
     public void visitVarDef(JCVariableDecl tree) {
         if (tree.sym == null) {
-            Logger.getLogger(Flow.class.getName()).warning("Flow.visitVarDef has a null tree.sym. Tree: [" + tree + "] Enclosing class: [" + classDef + "]"); //NOI18N
+            String variableName = String.valueOf(tree.name);
+            int variableStart = tree.getStartPosition();
+
+            if (classDef != null && classDef.sym != null) {
+                JavaFileObject file = classDef.sym.sourcefile != null ? classDef.sym.sourcefile : classDef.sym.classfile;
+                String source = file != null ? file.toUri().toASCIIString() : "<unknown>";
+                int classStart = classDef.getStartPosition();
+                String className = String.valueOf(classDef.sym.flatname);
+
+                Logger.getLogger(Flow.class.getName()).log(Level.WARNING, "Flow.visitVarDef has a null tree.sym. Variable name: {0}, offset: {1}, enclosing class name: {2}, offset: {3}, source file: {4}", new Object[] {variableName, variableStart, className, classStart, source}); //NOI18N
+            } else {
+                String className = classDef != null ? String.valueOf(classDef.name) : "<unknown>";
+
+                Logger.getLogger(Flow.class.getName()).log(Level.WARNING, "Flow.visitVarDef has a null tree.sym. Variable name: {0}, offset: {1}, enclosing class name: {2}", new Object[] {variableName, variableStart, className}); //NOI18N
+            }
+            Logger.getLogger(Flow.class.getName()).log(Level.FINER, "Flow.visitVarDef has a null tree.sym. Tree: [{0}] Enclosing class: [{1}]", new Object[] {tree, classDef}); //NOI18N
             return;
         }
         boolean track = trackable(tree.sym);
@@ -1271,6 +1304,11 @@ public class Flow extends TreeScanner {
         }
     }
 
+    public void visitAnnotatedType(JCAnnotatedType tree) {
+        // annotations don't get scanned
+        tree.underlyingType.accept(this);
+    }
+
     public void visitIdent(JCIdent tree) {
         if (tree.sym != null && tree.sym.kind == VAR)
             checkInit(tree.pos(), (VarSymbol)tree.sym);
@@ -1280,7 +1318,8 @@ public class Flow extends TreeScanner {
         super.visitTypeCast(tree);
         if (tree.type != null && !tree.type.isErroneous()
             && lint.isEnabled(Lint.LintCategory.CAST)
-            && types.isSameType(tree.expr.type, tree.clazz.type)) {
+            && types.isSameType(tree.expr.type, tree.clazz.type)
+            && !(ignoreAnnotatedCasts && containsTypeAnnotation(tree.clazz))) {
             log.warning(tree.pos(), "redundant.cast", tree.expr.type);
         }
     }
@@ -1289,6 +1328,23 @@ public class Flow extends TreeScanner {
         // Do nothing for TopLevel since each class is visited individually
     }
 
+/**************************************************************************
+ * utility methods for ignoring type-annotated casts lint checking
+ *************************************************************************/
+    private static final boolean ignoreAnnotatedCasts = true;
+    private static class AnnotationFinder extends TreeScanner {
+        public boolean foundTypeAnno = false;
+        public void visitAnnotation(JCAnnotation tree) {
+            foundTypeAnno = foundTypeAnno || (tree instanceof JCTypeAnnotation);
+        }
+    }
+
+    private boolean containsTypeAnnotation(JCTree e) {
+        AnnotationFinder finder = new AnnotationFinder();
+        finder.scan(e);
+        return finder.foundTypeAnno;
+    }
+
 /**************************************************************************
  * main method
  *************************************************************************/
