@@ -44,6 +44,7 @@ import static com.sun.tools.javac.code.Kinds.*;
 import static com.sun.tools.javac.code.TypeTags.*;
 import com.sun.tools.javac.model.LazyTreeLoader;
 import com.sun.tools.javac.util.JCDiagnostic.DiagnosticPosition;
+import javax.lang.model.element.ElementKind;
 import javax.lang.model.element.Modifier;
 
 /** This is the second phase of Enter, in which classes are completed
@@ -106,8 +107,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         diags = JCDiagnostic.Factory.instance(context);
         target = Target.instance(context);
         Options options = Options.instance(context);
-        skipAnnotations =
-            options.get("skipAnnotations") != null;
+        skipAnnotations = options.get("skipAnnotations") != null;
         boolean ideMode = options.get("ide") != null;
         boolean backgroundCompilation = options.get("backgroundCompilation") != null;
         ignoreNoLang = ideMode && !backgroundCompilation;
@@ -119,7 +119,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
     /** A queue for classes whose members still need to be entered into the
      *  symbol table.
      */
-    ListBuffer<Env<AttrContext>> halfcompleted = new ListBuffer<Env<AttrContext>>();
+    public ListBuffer<Env<AttrContext>> halfcompleted = new ListBuffer<Env<AttrContext>>();
 
     /** Set to true only when the first of a set of classes is
      *  processed from the halfcompleted queue.
@@ -442,12 +442,10 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
     void finishClass(JCClassDecl tree, Env<AttrContext> env) {
         if ((tree.mods.flags & Flags.ENUM) != 0 &&
             (types.supertype(tree.sym.type).tsym.flags() & Flags.ENUM) == 0) {
-            addEnumMembers(tree, env);
-            memberEnter(tree.defs, env);
-            chk.checkAllDefined(tree.pos(), tree.sym);
-        } else {
-            memberEnter(tree.defs, env);
+            if (tree.sym == null || (tree.sym.flags_field & Flags.FROMCLASS) == 0)
+                addEnumMembers(tree, env);
         }
+        memberEnter(tree.defs, env);
     }
 
     /** Add the implicit members for an enum type
@@ -614,63 +612,86 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                 if (e.sym.kind == MTH) {
                     boolean sameType = types.isSameType(m.type, e.sym.type);
                     if (sameType || m.name == names.init && (m.owner.name.isEmpty() || (m.owner.owner.kind & (VAR | MTH)) != 0)) {
-                        treeCleaner.scan(tree);
-                        tree.sym = (MethodSymbol)e.sym;
-                        localEnv = methodEnv(tree, env);
-                        tree.sym.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, tree.sym, tree);
-                        tree.sym.flags_field |= FROMCLASS;
-                        if (tree.sym.type.tag == FORALL) {
-                            for(List<Type> tvars = ((ForAll)tree.sym.type).tvars; tvars.nonEmpty(); tvars = tvars.tail)
-                                localEnv.info.scope.enter(tvars.head.tsym);
-                        }
-                        List<VarSymbol> p = tree.sym.params();
-                        if (p != null) {
-                            List<JCVariableDecl> l = tree.params;
-                            while(l.nonEmpty() && p.nonEmpty()) {
-                                if (sameType) {
-                                    p.head.setName(l.head.name);
-                                    if (l.head.getModifiers() != null && l.head.getModifiers().getFlags().contains(Modifier.FINAL)) {
-                                        //copy the final flag, as the symbol might have come from the classfile:
-                                        p.head.flags_field |= FINAL;
+                        if ((e.sym.flags_field & FROMCLASS) != 0) {
+                            treeCleaner.scan(tree);
+                            tree.sym = (MethodSymbol)e.sym;
+                            localEnv = methodEnv(tree, env);
+                            tree.sym.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, tree.sym, tree) | (tree.sym.flags_field & APT_CLEANED);
+                            tree.sym.flags_field |= FROMCLASS;
+                            if (tree.sym.type.tag == FORALL) {
+                                for(List<Type> tvars = ((ForAll)tree.sym.type).tvars; tvars.nonEmpty(); tvars = tvars.tail)
+                                    localEnv.info.scope.enter(tvars.head.tsym);
+                            }
+                            List<VarSymbol> p = tree.sym.params();
+                            if (p != null) {
+                                List<JCVariableDecl> l = tree.params;
+                                while(l.nonEmpty() && p.nonEmpty()) {
+                                    if (sameType) {
+                                        p.head.setName(l.head.name);
+                                        if (l.head.getModifiers() != null && l.head.getModifiers().getFlags().contains(Modifier.FINAL)) {
+                                            //copy the final flag, as the symbol might have come from the classfile:
+                                            p.head.flags_field |= FINAL;
+                                        }
                                     }
+                                    localEnv.info.scope.enter(p.head);
+                                    p.head.flags_field |= FROMCLASS;
+                                    p = p.tail;
+                                    l = l.tail;
+                                }
+                                while(p.nonEmpty()) {
+                                    p.head.setName(p.head.name);
+                                    localEnv.info.scope.enter(p.head);
+                                    p = p.tail;
                                 }
-                                localEnv.info.scope.enter(p.head);
-                                p = p.tail;
-                                l = l.tail;
                             }
-                            while(p.nonEmpty()) {
-                                p.head.setName(p.head.name);
-                                localEnv.info.scope.enter(p.head);
-                                p = p.tail;
+                            tree.sym.type = signature(tree.typarams, tree.params,
+                                    tree.restype, tree.thrown,
+                                    localEnv);
+                            tree.sym.flags_field &= ~FROMCLASS;
+                            localEnv.info.scope.leave();
+
+                            // Set m.params
+                            ListBuffer<VarSymbol> params = new ListBuffer<VarSymbol>();
+                            JCVariableDecl lastParam = null;
+                            for (List<JCVariableDecl> l = tree.params; l.nonEmpty(); l = l.tail) {
+                                JCVariableDecl param = lastParam = l.head;
+                                assert param.sym != null;
+                                params.append(param.sym);
                             }
+                            tree.sym.params = params.toList();
+
+                            // mark the method varargs, if necessary
+                            if (lastParam != null && (lastParam.mods.flags & Flags.VARARGS) != 0)
+                                tree.sym.flags_field |= Flags.VARARGS;
+                            tree.sym.flags_field &= ~APT_CLEANED;
                         }
-                        tree.sym.type = signature(tree.typarams, tree.params,
-                                tree.restype, tree.thrown,
-                                localEnv);
-                        tree.sym.flags_field &= ~FROMCLASS;
-                        localEnv.info.scope.leave();
-
-                        // Set m.params
-                        ListBuffer<VarSymbol> params = new ListBuffer<VarSymbol>();
-                        JCVariableDecl lastParam = null;
-                        for (List<JCVariableDecl> l = tree.params; l.nonEmpty(); l = l.tail) {
-                            JCVariableDecl param = lastParam = l.head;
-                            assert param.sym != null;
-                            params.append(param.sym);
-                        }
-                        tree.sym.params = params.toList();
-
-                        // mark the method varargs, if necessary
-                        if (lastParam != null && (lastParam.mods.flags & Flags.VARARGS) != 0)
-                            tree.sym.flags_field |= Flags.VARARGS;
-
                         break;
                     }
                 }
             }
             if (tree.sym == m) {
-                ClassSymbol cs = enclScope.owner.outermostClass();
-                treeLoader.couplingError(cs, tree);
+                if ((enclScope.owner.flags_field & APT_CLEANED) == 0) {
+                    ClassSymbol cs = enclScope.owner.outermostClass();
+                    treeLoader.couplingError(cs, tree);
+                } else {
+                    localEnv.info.scope.leave();
+                    // Set m.params
+                    ListBuffer<VarSymbol> params = new ListBuffer<VarSymbol>();
+                    JCVariableDecl lastParam = null;
+                    for (List<JCVariableDecl> l = tree.params; l.nonEmpty(); l = l.tail) {
+                        JCVariableDecl param = lastParam = l.head;
+                        assert param.sym != null;
+                        params.append(param.sym);
+                    }
+                    m.params = params.toList();
+
+                    // mark the method varargs, if necessary
+                    if (lastParam != null && (lastParam.mods.flags & Flags.VARARGS) != 0)
+                        m.flags_field |= Flags.VARARGS;
+
+                    if( chk.checkUnique(tree.pos(), m, enclScope))
+                        enclScope.enter(m);
+                }
             }
         } else {
             localEnv.info.scope.leave();
@@ -724,15 +745,20 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         if ((enclScope.owner.flags_field & FROMCLASS) != 0) {
             for (Scope.Entry e = enclScope.lookup(tree.name); e.scope == enclScope; e = e.next()) {
                 if (e.sym.kind == VAR && types.isSameType(tree.vartype.type, e.sym.type)) {
-                    v = (VarSymbol)e.sym;
+                    if ((e.sym.flags_field & FROMCLASS) != 0) {
+                        v = (VarSymbol)e.sym;
+                        e.sym.flags_field &= ~FROMCLASS;
+                    }
                     break;
                 }
             }
-            if (v == null) {
+            if (v != null) {
+                doEnterSymbol = false;
+            } else if ((enclScope.owner.flags_field & APT_CLEANED) == 0) {
                 ClassSymbol cs = enclScope.owner.outermostClass();
                 treeLoader.couplingError(cs, tree);
+                doEnterSymbol = false;
             }
-            doEnterSymbol = false;
         }
         if (v == null) {
             v = new VarSymbol(0, tree.name, tree.vartype.type, enclScope.owner);
@@ -747,10 +773,13 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                 v.setLazyConstValue(initEnv(tree, initEnv), log, attr, tree.init);
             }
         }
-        if (chk.checkUnique(tree.pos(), v, enclScope)) {
-            chk.checkTransparentVar(tree.pos(), v, enclScope);
-            if (doEnterSymbol)
+        if (doEnterSymbol) {
+            if (chk.checkUnique(tree.pos(), v, enclScope)) {
+                chk.checkTransparentVar(tree.pos(), v, enclScope);
                 enclScope.enter(v);
+            } else if (v.getKind() == ElementKind.LOCAL_VARIABLE) {
+                enclScope.enter(v);
+            }
         }
         annotateLater(tree.mods.annotations, localEnv, v);
         v.pos = tree.pos;
@@ -766,7 +795,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
      */
     Env<AttrContext> initEnv(JCVariableDecl tree, Env<AttrContext> env) {
         Env<AttrContext> localEnv = env.dupto(new AttrContextEnv(tree, env.info.dup()));
-        if (tree.sym.owner.kind == TYP) {
+        if (tree.sym.owner.kind == TYP || tree.sym.owner.kind == ERR) {
             localEnv.info.scope = new Scope.DelegatedScope(env.info.scope);
             localEnv.info.scope.owner = tree.sym;
         }
@@ -831,13 +860,13 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                        final Env<AttrContext> localEnv,
                        final Symbol s) {
         if (annotations.isEmpty()) return;
-        if (s.kind != PCK) s.attributes_field = null; // mark it incomplete for now
+        if (s.kind != PCK || (s.flags_field & APT_CLEANED) != 0) s.attributes_field = null; // mark it incomplete for now
         annotate.later(new Annotate.Annotator() {
                 public String toString() {
                     return "annotate " + annotations + " onto " + s + " in " + s.owner;
                 }
                 public void enterAnnotation() {
-                    assert s.kind == PCK || s.attributes_field == null;
+                    if (s.kind != PCK) s.attributes_field = null;
                     JavaFileObject prev = log.useSource(localEnv.toplevel.sourcefile);
                     try {
                         if (s.attributes_field != null &&
@@ -958,6 +987,9 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
             // clauses have been seen.
             if (c.owner.kind == PCK) {
                 memberEnter(env.toplevel, env.enclosing(JCTree.TOPLEVEL));
+                if ((c.flags_field & APT_CLEANED) != 0) {
+                    todo.remove(c);
+                }
                 todo.append(env);
             }
 
@@ -1026,6 +1058,10 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
             if (hasDeprecatedAnnotation(tree.mods.annotations))
                 c.flags_field |= DEPRECATED;
             annotateLater(tree.mods.annotations, baseEnv, c);
+            // class type parameters use baseEnv but everything uses env
+            for (JCTypeParameter tp : tree.typarams)
+                tp.accept(new TypeAnnotate(baseEnv));
+            tree.accept(new TypeAnnotate(env));
 
             chk.checkNonCyclic(tree.pos(), c.type);
 
@@ -1112,6 +1148,91 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         }
     }
 
+    // A sub-phase that "compiles" annotations in annotated types.
+    private class TypeAnnotate extends TreeScanner {
+        private Env<AttrContext> env;
+        public TypeAnnotate(Env<AttrContext> env) { this.env = env; }
+
+        private void enterTypeAnnotations(List<JCTypeAnnotation> annotations) {
+            Set<TypeSymbol> annotated = new HashSet<TypeSymbol>();
+            if (!skipAnnotations)
+                for (List<JCTypeAnnotation> al = annotations; al.nonEmpty(); al = al.tail) {
+                    JCTypeAnnotation a = al.head;
+                    Attribute.Compound c = annotate.enterAnnotation(a,
+                            syms.annotationType,
+                            env);
+                    if (c == null) continue;
+                    Attribute.TypeCompound tc = new Attribute.TypeCompound(c.type, c.values, a.annotation_position);
+                    a.attribute_field = tc;
+                    // Note: @Deprecated has no effect on local variables and parameters
+                    if (!annotated.add(a.type.tsym))
+                        log.error(a.pos, "duplicate.annotation");
+                }
+        }
+
+        // each class (including enclosed inner classes) should be visited
+        // separately through MemberEnter.complete(Symbol)
+        // this flag is used to prevent from visiting inner classes.
+        private boolean isEnclosingClass = false;
+        @Override
+        public void visitClassDef(final JCClassDecl tree) {
+            if (isEnclosingClass)
+                return;
+            isEnclosingClass = true;
+            scan(tree.mods);
+            // type parameter need to be visited with a separate env
+            // scan(tree.typarams);
+            scan(tree.extending);
+            scan(tree.implementing);
+            scan(tree.defs);
+        }
+
+        private void annotate(final JCTree tree, final List<JCTypeAnnotation> annotations) {
+            annotate.later(new Annotate.Annotator() {
+                public String toString() {
+                    return "annotate " + annotations + " onto " + tree;
+                }
+                public void enterAnnotation() {
+                    JavaFileObject prev = log.useSource(env.toplevel.sourcefile);
+                    try {
+                        enterTypeAnnotations(annotations);
+                    } finally {
+                        log.useSource(prev);
+                    }
+                }
+            });
+        }
+
+        @Override
+        public void visitAnnotatedType(final JCAnnotatedType tree) {
+            annotate(tree, tree.annotations);
+            super.visitAnnotatedType(tree);
+        }
+        @Override
+        public void visitTypeParameter(final JCTypeParameter tree) {
+            annotate(tree, tree.annotations);
+            super.visitTypeParameter(tree);
+        }
+        @Override
+        public void visitNewArray(final JCNewArray tree) {
+            annotate(tree, tree.annotations);
+            for (List<JCTypeAnnotation> dimAnnos : tree.dimAnnotations)
+                annotate(tree, dimAnnos);
+            super.visitNewArray(tree);
+        }
+        @Override
+        public void visitApply(JCMethodInvocation tree) {
+            super.visitApply(tree);
+            scan(tree.typeargs);
+        }
+        @Override
+        public void visitMethodDef(JCMethodDecl tree) {
+            annotate(tree, tree.receiverAnnotations);
+            super.visitMethodDef(tree);
+        }
+    }
+
+
     private Env<AttrContext> baseEnv(JCClassDecl tree, Env<AttrContext> env) {
         Scope typaramScope = new Scope(tree.sym);
         if (tree.typarams != null)
