@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2008 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright 1999-2009 Sun Microsystems, Inc.  All Rights Reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,6 +26,7 @@
 package com.sun.tools.javac.main;
 
 import java.io.*;
+import java.util.Collections;
 import java.util.HashSet;
 import java.util.LinkedHashSet;
 import java.util.LinkedHashMap;
@@ -37,6 +38,7 @@ import java.util.logging.Handler;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
+import javax.tools.Diagnostic;
 import javax.tools.JavaFileManager;
 import javax.tools.JavaFileObject;
 import javax.tools.DiagnosticListener;
@@ -294,10 +296,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      */
     protected ParserFactory parserFactory;
 
-    /** Optional listener for progress events
-     */
-    protected TaskListener taskListener;
-
 
     protected FlowListener flowListener;
     
@@ -323,6 +321,11 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
 
     protected Map<JavaFileObject, JCCompilationUnit> notYetEntered;
 
+    public boolean skipAnnotationProcessing = false;
+
+    private MemberEnter memberEnter;
+    public List<JCCompilationUnit> toProcessAnnotations = List.nil();
+
     /** Construct a new compiler using a shared context.
      */
     public JavaCompiler(final Context context) {
@@ -340,6 +343,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         make = TreeMaker.instance(context);
         writer = ClassWriter.instance(context);
         enter = Enter.instance(context);
+        memberEnter = MemberEnter.instance(context);
         todo = Todo.instance(context);
         flowListener = FlowListener.instance(context);
         memoryWatch = LowMemoryWatch.instance(context);
@@ -366,7 +370,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         lower = Lower.instance(context);
         annotate = Annotate.instance(context);
         types = Types.instance(context);
-        taskListener = context.get(TaskListener.class);
 
         reader.sourceCompleter = this;
 
@@ -400,6 +403,13 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             (options.get("failcomplete") != null)
             ? names.fromString(options.get("failcomplete"))
             : null;
+
+        shouldStopPolicy =
+            (options.get("shouldStopPolicy") != null)
+            ? CompileState.valueOf(options.get("shouldStopPolicy"))
+            : CompileState.GENERATE;
+        if (options.get("oldDiags") == null)
+            log.setDiagnosticFormatter(RichDiagnosticFormatter.instance(context));
     }
 
     /* Switches:
@@ -473,14 +483,26 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      */
     public boolean verboseCompilePolicy;
 
+    /**
+     * Policy of how far to continue processing. null means until first
+     * error.
+     */
+    public CompileState shouldStopPolicy;
+
     /** A queue of all as yet unattributed classes.
      */
     public Todo todo;
 
-    protected enum CompileState {
-        TODO(0),
-        ATTR(1),
-        FLOW(2);
+    /** Ordered list of compiler phases for each compilation unit. */
+    public enum CompileState {
+        PARSE(1),
+        ENTER(2),
+        PROCESS(3),
+        ATTR(4),
+        FLOW(5),
+        TRANSTYPES(6),
+        LOWER(7),
+        GENERATE(8);
         CompileState(int value) {
             this.value = value;
         }
@@ -489,6 +511,9 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         }
         private int value;
     };
+    /** Partial map to record which compiler phases have been executed
+     * for each compilation unit. Used for ATTR and FLOW phases.
+     */
     protected class CompileStates extends HashMap<Env<AttrContext>,CompileState> {
         private static final long serialVersionUID = 1812267524140424433L;
         boolean isDone(Env<AttrContext> env, CompileState cs) {
@@ -504,6 +529,13 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      */
     protected Set<JavaFileObject> inputFiles = new HashSet<JavaFileObject>();
 
+    protected boolean shouldStop(CompileState cs) {
+        if (shouldStopPolicy == null)
+            return (errorCount() > 0);
+        else
+            return cs.ordinal() > shouldStopPolicy.ordinal();
+    }
+
     /** The number of errors reported so far.
      */
     public int errorCount() {
@@ -517,18 +549,12 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             return log.nerrors;
     }
 
-    protected final <T> Queue<T> stopIfError(Queue<T> queue) {
-        if (errorCount() == 0)
-            return queue;
-        else
-            return ListBuffer.lb();
+    protected final <T> Queue<T> stopIfError(CompileState cs, Queue<T> queue) {
+        return shouldStop(cs) ? ListBuffer.<T>lb() : queue;
     }
 
-    protected final <T> List<T> stopIfError(List<T> list) {
-        if (errorCount() == 0)
-            return list;
-        else
-            return List.nil();
+    protected final <T> List<T> stopIfError(CompileState cs, List<T> list) {
+        return shouldStop(cs) ? List.<T>nil() : list;
     }
 
     /** The number of warnings reported so far.
@@ -572,10 +598,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             if (verbose) {
                 printVerbose("parsing.started", filename);
             }
-            if (taskListener != null) {
-                TaskEvent e = new TaskEvent(TaskEvent.Kind.PARSE, filename);
-                taskListener.started(e);
-            }
+            taskStarted(new TaskEvent(TaskEvent.Kind.PARSE, filename));
             int initialErrorCount = log.nerrors;
             Parser parser = parserFactory.newParser(content, keepComments(), genEndPos, lineDebugInfo);
             tree = parser.parseCompilationUnit();
@@ -587,9 +610,8 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
 
         tree.sourcefile = filename;
 
-        if (content != null && taskListener != null) {
-            TaskEvent e = new TaskEvent(TaskEvent.Kind.PARSE, tree);
-            taskListener.finished(e);
+        if (content != null) {
+            taskFinished(new TaskEvent(TaskEvent.Kind.PARSE, tree));
         }
 
         return tree;
@@ -600,6 +622,32 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             return keepComments || sourceOutput || stubOutput;
         }
 
+    private List<Diagnostic<? extends JavaFileObject>> tempDiags = List.nil();
+    private Set<Pair<JavaFileObject, Integer>> recordedOverlay;
+    private Map<JCTree, JCDiagnostic> errTreesOverlay;
+    private DiagnosticListener<? super JavaFileObject> temporaryDiagListener = new DiagnosticListener<JavaFileObject>() {
+        public void report(Diagnostic<? extends JavaFileObject> diagnostic) {
+            tempDiags = tempDiags.prepend(diagnostic);
+        }
+    };
+    public void flushTempDiags() {
+        DiagnosticListener<? super JavaFileObject> diagnosticListener = log.getDiagnosticListener();
+        if (diagnosticListener != null) {
+            tempDiags = tempDiags.reverse();
+            for (Diagnostic<? extends JavaFileObject> diagnostic : tempDiags) {
+                diagnosticListener.report(diagnostic);
+            }
+        }
+        if (recordedOverlay != null) {
+            log.pushRecordedOverlay(recordedOverlay);
+            recordedOverlay = null;
+        }
+        if (errTreesOverlay != null) {
+            log.pushErrTreesOverlay(errTreesOverlay);
+            errTreesOverlay = null;
+        }
+        tempDiags = List.nil();
+    }
 
     /** Parse contents of file.
      *  @param filename     The name of the file to be parsed.
@@ -632,6 +680,8 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         if (name.equals(""))
             return syms.errSymbol;
         JavaFileObject prev = log.useSource(null);
+        boolean old = log.suppressErrorsAndWarnings;
+        log.suppressErrorsAndWarnings = true;
         try {
             JCExpression tree = null;
             for (String s : name.split("\\.", -1)) {
@@ -645,6 +695,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             toplevel.packge = syms.unnamedPackage;
             return attr.attribIdent(tree, toplevel);
         } finally {
+            log.suppressErrorsAndWarnings = old;
             log.useSource(prev);
         }
     }
@@ -724,16 +775,25 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             log.useSource(prev);
         }
 
-        if (taskListener != null) {
-            TaskEvent e = new TaskEvent(TaskEvent.Kind.ENTER, tree);
-            taskListener.started(e);
-        }
+        taskStarted(new TaskEvent(TaskEvent.Kind.ENTER, tree));
 
-        enter.complete(List.of(tree), c);
+        complete(List.of(tree), c);
 
-        if (taskListener != null) {
-            TaskEvent e = new TaskEvent(TaskEvent.Kind.ENTER, tree);
-            taskListener.finished(e);
+        taskFinished(new TaskEvent(TaskEvent.Kind.ENTER, tree));
+
+        if (!skipAnnotationProcessing) {
+            if (memberEnter.halfcompleted.nonEmpty() || annotate.enterCount > 0) {
+                toProcessAnnotations = toProcessAnnotations.prepend(tree);
+            } else {
+                try {
+                    processAnnotations(List.of(tree));
+                    flushTempDiags();
+                } catch (IOException ex) {
+                    CompletionFailure cf = new CompletionFailure(c, ex.getLocalizedMessage());
+                    cf.initCause(ex);
+                    throw cf;
+                }
+            }
         }
 
         if (enter.getEnv(c) == null) {
@@ -797,15 +857,21 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             initProcessAnnotations(processors);
 
             // These method calls must be chained to avoid memory leaks
-            delegateCompiler = processAnnotations(enterTrees(stopIfError(parseFiles(sourceFileObjects))),
-                                                  classnames);
+            delegateCompiler =
+                processAnnotations(
+                    enterTrees(stopIfError(CompileState.PARSE, parseFiles(sourceFileObjects))),
+                    classnames);
 
+            delegateCompiler.flushTempDiags();
             delegateCompiler.compile2();
             delegateCompiler.close();
             elapsed_msec = delegateCompiler.elapsed_msec;
         } catch (Abort ex) {
             if (devVerbose)
                 ex.printStackTrace();
+        } finally {
+            if (procEnvImpl != null)
+                procEnvImpl.close();
         }
     }
 
@@ -830,7 +896,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
 
             case BY_FILE: {
                     Queue<Queue<Env<AttrContext>>> q = todo.groupByFile();
-                    while (!q.isEmpty() && errorCount() == 0) {
+                    while (!q.isEmpty() && !shouldStop(CompileState.ATTR)) {
                         generate(desugar(flow(attribute(q.remove()))));
                     }
                 }
@@ -868,10 +934,8 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      * Parses a list of files.
      */
    public List<JCCompilationUnit> parseFiles(List<JavaFileObject> fileObjects) throws IOException {
-       if (errorCount() > 0) {
-           Logger.getLogger(JavaCompiler.class.getName()).warning("parseFiles stopped due to previous errors");
+       if (shouldStop(CompileState.PARSE))
            return List.nil();
-       }
 
         //parse all files
         ListBuffer<JCCompilationUnit> trees = lb();
@@ -889,20 +953,14 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      */
     public List<JCCompilationUnit> enterTrees(List<JCCompilationUnit> roots) {
         //enter symbols for all files
-        if (taskListener != null) {
-            for (JCCompilationUnit unit: roots) {
-                TaskEvent e = new TaskEvent(TaskEvent.Kind.ENTER, unit);
-                taskListener.started(e);
-            }
+        for (JCCompilationUnit unit : roots) {
+            taskStarted(new TaskEvent(TaskEvent.Kind.ENTER, unit));
         }
 
-        enter.main(roots);
+        complete(roots, null);
 
-        if (taskListener != null) {
-            for (JCCompilationUnit unit: roots) {
-                TaskEvent e = new TaskEvent(TaskEvent.Kind.ENTER, unit);
-                taskListener.finished(e);
-            }
+        for (JCCompilationUnit unit : roots) {
+            taskFinished(new TaskEvent(TaskEvent.Kind.ENTER, unit));
         }
 
         //If generating source, remember the classes declared in
@@ -911,10 +969,11 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             ListBuffer<JCClassDecl> cdefs = lb();
             for (JCCompilationUnit unit : roots) {
                 for (List<JCTree> defs = unit.defs;
-                     defs.nonEmpty();
-                     defs = defs.tail) {
-                    if (defs.head instanceof JCClassDecl)
-                        cdefs.append((JCClassDecl)defs.head);
+                        defs.nonEmpty();
+                        defs = defs.tail) {
+                    if (defs.head instanceof JCClassDecl) {
+                        cdefs.append((JCClassDecl) defs.head);
+                    }
                 }
             }
             rootClasses = cdefs.toList();
@@ -922,6 +981,21 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         return roots;
     }
 
+    private void complete(List<JCCompilationUnit> roots, ClassSymbol c) {
+        DiagnosticListener<? super JavaFileObject> oldDiagListener = log.getDiagnosticListener();
+        tempDiags = List.nil();
+        log.setRecordedOverlay(recordedOverlay = new HashSet<Pair<JavaFileObject, Integer>>());
+        log.setErrTreesOverlay(errTreesOverlay = new HashMap<JCTree, JCDiagnostic>());
+        log.setDiagnosticListener(temporaryDiagListener);
+        try {
+            enter.complete(roots, c);
+        } finally {
+            log.setRecordedOverlay(null);
+            log.setErrTreesOverlay(null);
+            log.setDiagnosticListener(oldDiagListener);
+        }
+    }
+
     public void initNotYetEntered(Map<JavaFileObject, JCCompilationUnit> notYetEntered) {
         this.notYetEntered = notYetEntered;
     }
@@ -937,7 +1011,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
     /**
      * Object to handle annotation processing.
      */
-    JavacProcessingEnvironment procEnvImpl = null;
+    private JavacProcessingEnvironment procEnvImpl = null;
 
     /**
      * Check if we should process annotations.
@@ -948,7 +1022,8 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      * @param processors user provided annotation processors to bypass
      * discovery, {@code null} means that no processors were provided
      */
-    public void initProcessAnnotations(Iterable<? extends Processor> processors) {
+    public void initProcessAnnotations(Iterable<? extends Processor> processors)
+                throws IOException {
         // Process annotations if processing is not disabled and there
         // is at least one Processor available.
         Options options = Options.instance(context);
@@ -964,8 +1039,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                 options.put("save-parameter-names", "save-parameter-names");
                 reader.saveParameterNames = true;
                 keepComments = true;
-                if (taskListener != null)
-                    taskListener.started(new TaskEvent(TaskEvent.Kind.ANNOTATION_PROCESSING));
+                taskStarted(new TaskEvent(TaskEvent.Kind.ANNOTATION_PROCESSING));
 
 
             } else { // free resources
@@ -975,7 +1049,8 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
     }
 
     // TODO: called by JavacTaskImpl
-    public JavaCompiler processAnnotations(List<JCCompilationUnit> roots) throws IOException {
+    public JavaCompiler processAnnotations(List<JCCompilationUnit> roots)
+            throws IOException {
         return processAnnotations(roots, List.<String>nil());
     }
 
@@ -987,7 +1062,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
     public JavaCompiler processAnnotations(List<JCCompilationUnit> roots,
                                            List<String> classnames)
         throws IOException  { // TODO: see TEMP note in JavacProcessingEnvironment
-        if (errorCount() != 0) {
+        if (shouldStop(CompileState.PROCESS)) {
             // Errors were encountered.  If todo is empty, then the
             // encountered errors were parse errors.  Otherwise, the
             // errors were found during the enter phase which should
@@ -1062,10 +1137,33 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                         return this;
                 }
             }
-            JavaCompiler c = procEnvImpl.doProcessing(context, roots, classSymbols, pckSymbols);
-            if (c != this)
-                annotationProcessingOccurred = c.annotationProcessingOccurred = true;
-            return c;
+            try {
+                JavaCompiler c = this;
+                if (roots.nonEmpty()) {
+                    fileManager.handleOption("apt-origin", Collections.singleton(roots.head.getSourceFile().toUri().toString()).iterator());    //NOI18N
+                    try {
+                        c = procEnvImpl.doProcessing(context, roots, classSymbols, pckSymbols);
+                    } finally {
+                        fileManager.handleOption("apt-origin", Collections.singletonList("").iterator());   //NOI18N
+                    }
+                }
+                List<JCCompilationUnit> currentAnnotations = toProcessAnnotations;
+                toProcessAnnotations = List.nil();
+                while(currentAnnotations.nonEmpty()) {
+                    fileManager.handleOption("apt-origin", Collections.singleton(currentAnnotations.head.getSourceFile().toUri().toString()).iterator()); //NOI18N
+                    try {
+                        procEnvImpl.doProcessing(context, List.of(currentAnnotations.head), classSymbols, pckSymbols);
+                    } finally {
+                        fileManager.handleOption("apt-origin", Collections.singletonList("").iterator());   //NOI18N
+                    }
+                    currentAnnotations = currentAnnotations.tail;
+                }
+                if (c != this)
+                    annotationProcessingOccurred = c.annotationProcessingOccurred = true;
+                return c;
+            } finally {
+                procEnvImpl.close(false);
+            }
         } catch (CompletionFailure ex) {
             log.error("cant.access", ex.sym, ex.getDetailValue());
             return this;
@@ -1096,7 +1194,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             results.append(attribute(envs.remove()));
             memoryWatch.abortIfMemoryLow();
         }
-        return results;
+        return stopIfError(CompileState.ATTR, results);
     }
 
     /**
@@ -1112,10 +1210,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         if (verbose)
             printVerbose("checking.attribution", env.enclClass.sym);
 
-        if (taskListener != null) {
-            TaskEvent e = new TaskEvent(TaskEvent.Kind.ANALYZE, env.toplevel, env.enclClass.sym);
-            taskListener.started(e);
-        }
+        taskStarted(new TaskEvent(TaskEvent.Kind.ANALYZE, env.toplevel, env.enclClass.sym));
 
         JavaFileObject prev = log.useSource(
                                   env.enclClass.sym.sourcefile != null ?
@@ -1144,7 +1239,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             flow(env, results);
             memoryWatch.abortIfMemoryLow();
         }
-        return results;
+        return stopIfError(CompileState.FLOW, results);
     }
 
     /**
@@ -1153,7 +1248,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
     public Queue<Env<AttrContext>> flow(Env<AttrContext> env) {
         ListBuffer<Env<AttrContext>> results = lb();
         flow(env, results);
-        return results;
+        return stopIfError(CompileState.FLOW, results);
     }
 
     /**
@@ -1161,13 +1256,16 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      */
     protected void flow(Env<AttrContext> env, Queue<Env<AttrContext>> results) {
         try {
+            if (shouldStop(CompileState.FLOW))
+                return;
+
             if (relax || compileStates.isDone(env, CompileState.FLOW)) {
                 results.add(env);
                 return;
             }
 
             if (verboseCompilePolicy)
-                log.printLines(log.noticeWriter, "[flow " + env.enclClass.sym + "]");
+                printNote("[flow " + env.enclClass.sym + "]");
             JavaFileObject prev = log.useSource(
                                                 env.enclClass.sym.sourcefile != null ?
                                                 env.enclClass.sym.sourcefile :
@@ -1184,6 +1282,9 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                 }
                 compileStates.put(env, CompileState.FLOW);
 
+                if (shouldStop(CompileState.FLOW))
+                    return;
+
                 results.add(env);
             }
             finally {
@@ -1191,10 +1292,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             }
         }
         finally {
-            if (taskListener != null) {
-                TaskEvent e = new TaskEvent(TaskEvent.Kind.ANALYZE, env.toplevel, env.enclClass.sym);
-                taskListener.finished(e);
-            }
+            taskFinished(new TaskEvent(TaskEvent.Kind.ANALYZE, env.toplevel, env.enclClass.sym));
         }
     }
 
@@ -1212,9 +1310,12 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             desugar(env, results);
             memoryWatch.abortIfMemoryLow();
         }
-        return results;
+        return stopIfError(CompileState.FLOW, results);
     }
 
+    HashMap<Env<AttrContext>, Queue<Pair<Env<AttrContext>, JCClassDecl>>> desugaredEnvs =
+            new HashMap<Env<AttrContext>, Queue<Pair<Env<AttrContext>, JCClassDecl>>>();
+
     /**
      * Prepare attributed parse trees, in conjunction with their attribution contexts,
      * for source or code generation. If the file was not listed on the command line,
@@ -1222,26 +1323,39 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
      * The preparation stops as soon as an error is found.
      */
     protected void desugar(final Env<AttrContext> env, Queue<Pair<Env<AttrContext>, JCClassDecl>> results) {
+        if (shouldStop(CompileState.TRANSTYPES))
+            return;
+
         if (implicitSourcePolicy == ImplicitSourcePolicy.NONE
                 && !inputFiles.contains(env.toplevel.sourcefile)) {
             return;
         }
 
+        if (compileStates.isDone(env, CompileState.LOWER)) {
+            results.addAll(desugaredEnvs.get(env));
+            return;
+        }
+
         /**
-         * As erasure (TransTypes) destroys information needed in flow analysis,
-         * including information in supertypes, we need to ensure that supertypes
-         * are processed through attribute and flow before subtypes are translated.
+         * Ensure that superclasses of C are desugared before C itself. This is
+         * required for two reasons: (i) as erasure (TransTypes) destroys
+         * information needed in flow analysis and (ii) as some checks carried
+         * out during lowering require that all synthetic fields/methods have
+         * already been added to C and its superclasses.
          */
         class ScanNested extends TreeScanner {
             Set<Env<AttrContext>> dependencies = new LinkedHashSet<Env<AttrContext>>();
+            @Override
             public void visitClassDef(JCClassDecl node) {
-                Type st = types.supertype(node.sym.type);
-                if (st.tag == TypeTags.CLASS) {
-                    ClassSymbol c = st.tsym.outermostClass();
-                    Env<AttrContext> stEnv = enter.getEnv(c);
-                    if (stEnv != null && env != stEnv) {
-                        if (dependencies.add(stEnv))
-                            scan(stEnv.tree);
+                if (node.sym != null) {
+                    Type st = types.supertype(node.sym.type);
+                    if (st.tag == TypeTags.CLASS) {
+                        ClassSymbol c = st.tsym.outermostClass();
+                        Env<AttrContext> stEnv = enter.getEnv(c);
+                        if (stEnv != null && env != stEnv) {
+                            if (dependencies.add(stEnv))
+                                scan(stEnv.tree);
+                        }
                     }
                 }
                 super.visitClassDef(node);
@@ -1250,12 +1364,17 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         ScanNested scanner = new ScanNested();
         scanner.scan(env.tree);
         for (Env<AttrContext> dep: scanner.dependencies) {
-            if (!compileStates.isDone(dep, CompileState.FLOW))
-                flow(attribute(dep));
+        if (!compileStates.isDone(dep, CompileState.FLOW))
+            desugaredEnvs.put(dep, desugar(flow(attribute(dep))));
         }
 
+        //We need to check for error another time as more classes might
+        //have been attributed and analyzed at this stage
+        if (shouldStop(CompileState.TRANSTYPES))
+            return;
+
         if (verboseCompilePolicy)
-            log.printLines(log.noticeWriter, "[desugar " + env.enclClass.sym + "]");
+            printNote("[desugar " + env.enclClass.sym + "]");
 
         JavaFileObject prev = log.useSource(env.enclClass.sym.sourcefile != null ?
                                   env.enclClass.sym.sourcefile :
@@ -1269,6 +1388,8 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
 
             if (env.tree instanceof JCCompilationUnit) {
                 if (!(stubOutput || sourceOutput || printFlat)) {
+                    if (shouldStop(CompileState.LOWER))
+                        return;
                     List<JCTree> pdef = lower.translateTopLevelClass(env, env.tree, localMake);
                     if (pdef.head != null) {
                         assert pdef.tail.isEmpty();
@@ -1294,7 +1415,14 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                 return;
             }
 
+            if (shouldStop(CompileState.TRANSTYPES))
+                return;
+
             env.tree = transTypes.translateTopLevelClass(env.tree, localMake);
+            compileStates.put(env, CompileState.TRANSTYPES);
+
+            if (shouldStop(CompileState.LOWER))
+                return;
 
             if (sourceOutput) {
                 //emit standard Java source file, only for compilation
@@ -1309,6 +1437,10 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
 
             //translate out inner classes
             List<JCTree> cdefs = lower.translateTopLevelClass(env, env.tree, localMake);
+            compileStates.put(env, CompileState.LOWER);
+
+            if (shouldStop(CompileState.LOWER))
+                return;
 
             //generate code for each class
             for (List<JCTree> l = cdefs; l.nonEmpty(); l = l.tail) {
@@ -1332,6 +1464,9 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
     }
 
     public void generate(Queue<Pair<Env<AttrContext>, JCClassDecl>> queue, Queue<JavaFileObject> results) {
+        if (shouldStop(CompileState.GENERATE))
+            return;
+
         boolean usePrintSource = (stubOutput || sourceOutput || printFlat);
 
         for (Pair<Env<AttrContext>, JCClassDecl> x: queue) {
@@ -1339,15 +1474,12 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             JCClassDecl cdef = x.snd;
 
             if (verboseCompilePolicy) {
-                log.printLines(log.noticeWriter, "[generate "
+                printNote("[generate "
                                + (usePrintSource ? " source" : "code")
                                + " " + cdef.sym + "]");
             }
 
-            if (taskListener != null) {
-                TaskEvent e = new TaskEvent(TaskEvent.Kind.GENERATE, env.toplevel, cdef.sym);
-                taskListener.started(e);
-            }
+            taskStarted(new TaskEvent(TaskEvent.Kind.GENERATE, env.toplevel, cdef.sym));
 
             JavaFileObject prev = log.useSource(env.enclClass.sym.sourcefile != null ?
                                       env.enclClass.sym.sourcefile :
@@ -1368,10 +1500,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                 log.useSource(prev);
             }
 
-            if (taskListener != null) {
-                TaskEvent e = new TaskEvent(TaskEvent.Kind.GENERATE, env.toplevel, cdef.sym);
-                taskListener.finished(e);
-            }
+            taskFinished(new TaskEvent(TaskEvent.Kind.GENERATE, env.toplevel, cdef.sym));
             
             memoryWatch.abortIfMemoryLow();
         }
@@ -1395,6 +1524,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         JCClassDecl removeMethodBodies(JCClassDecl cdef) {
             final boolean isInterface = (cdef.mods.flags & Flags.INTERFACE) != 0;
             class MethodBodyRemover extends TreeTranslator {
+                @Override
                 public void visitMethodDef(JCMethodDecl tree) {
                     tree.mods.flags &= ~Flags.SYNCHRONIZED;
                     for (JCVariableDecl vd : tree.params)
@@ -1402,11 +1532,13 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                     tree.body = null;
                     super.visitMethodDef(tree);
                 }
+                @Override
                 public void visitVarDef(JCVariableDecl tree) {
                     if (tree.init != null && tree.init.type.constValue() == null)
                         tree.init = null;
                     super.visitVarDef(tree);
                 }
+                @Override
                 public void visitClassDef(JCClassDecl tree) {
                     ListBuffer<JCTree> newdefs = lb();
                     for (List<JCTree> it = tree.defs; it.tail != null; it = it.tail) {
@@ -1484,6 +1616,9 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         log.flush();
         try {
             fileManager.flush();
+            if (procEnvImpl != null)
+                procEnvImpl.close(true);
+            procEnvImpl = null;
         } catch (IOException e) {
             throw new Abort(e);
         } finally {
@@ -1493,12 +1628,16 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         }
     }
 
+    protected void printNote(String lines) {
+        Log.printLines(log.noticeWriter, lines);
+    }
+
     /** Output for "-verbose" option.
      *  @param key The key to look up the correct internationalized string.
      *  @param arg An argument for substitution into the output string.
      */
     protected void printVerbose(String key, Object arg) {
-        Log.printLines(log.noticeWriter, log.getLocalizedString("verbose." + key, arg));
+        Log.printLines(log.noticeWriter, Log.getLocalizedString("verbose." + key, arg));
     }
 
     /** Print numbers of errors and warnings.
@@ -1507,9 +1646,9 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         if (count != 0) {
             String text;
             if (count == 1)
-                text = log.getLocalizedString("count." + kind, String.valueOf(count));
+                text = Log.getLocalizedString("count." + kind, String.valueOf(count));
             else
-                text = log.getLocalizedString("count." + kind + ".plural", String.valueOf(count));
+                text = Log.getLocalizedString("count." + kind + ".plural", String.valueOf(count));
             Log.printLines(log.errWriter, text);
             log.errWriter.flush();
         }
@@ -1526,6 +1665,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
     public void initRound(JavaCompiler prev) {
         keepComments = prev.keepComments;
         start_msec = prev.start_msec;
+        notYetEntered = prev.notYetEntered;
         hasBeenUsed = true;
     }
 
@@ -1537,4 +1677,20 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
        }
 
     }
+
+    private void taskStarted(TaskEvent evt) {
+        TaskListener tl = context.get(TaskListener.class);
+
+        if (tl != null) {
+            tl.started(evt);
+        }
+    }
+
+    private void taskFinished(TaskEvent evt) {
+        TaskListener tl = context.get(TaskListener.class);
+
+        if (tl != null) {
+            tl.finished(evt);
+        }
+    }
 }
