@@ -38,6 +38,11 @@ import com.sun.tools.javac.util.Options;
 
 /**
  * A factory for creating parsers.
+ *
+ * <p><b>This is NOT part of any API supported by Sun Microsystems.
+ * If you write code that depends on this, you do so at your own risk.
+ * This code and its internal interfaces are subject to change or
+ * deletion without notice.</b>
  */
 public class ParserFactory {
 
@@ -78,11 +83,10 @@ public class ParserFactory {
         return newParser (input, keepDocComments, keepEndPos, keepLineMap, false);
     }
 
-    public Parser newParser(CharSequence input, int startPos, int firstAnnonClassIndex, Map<JCTree,Integer> endPos) {
+    public Parser newParser(CharSequence input, int startPos, Map<JCTree,Integer> endPos) {
         Lexer lexer = scannerFactory.newScanner(input);
         ((Scanner)lexer).seek(startPos);
         JavacParser p = new EndPosParser(this, lexer, true, false, cancelSevice, endPos);
-        p.newAnonScope(names.empty, firstAnnonClassIndex);
         return p;
     }
 
@@ -94,9 +98,6 @@ public class ParserFactory {
         } else {
             p = new JavacParser(this, lexer, keepDocComments, keepLineMap, cancelSevice);
         }
-        if (partial) {
-            p.newAnonScope(names.empty, -1);
-        }
         return p;
     }
 }
\ No newline at end of file
