@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2008 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright 2005-2009 Sun Microsystems, Inc.  All Rights Reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,8 +26,12 @@
 package com.sun.tools.javac.processing;
 
 
+import com.sun.source.util.TreePath;
 import java.lang.reflect.*;
 import java.util.*;
+import java.util.Map.Entry;
+import java.util.logging.Level;
+import java.util.logging.Logger;
 import java.util.regex.*;
 
 import java.net.URL;
@@ -41,30 +45,35 @@ import java.io.StringWriter;
 import javax.annotation.processing.*;
 import javax.lang.model.SourceVersion;
 import javax.lang.model.element.AnnotationMirror;
+import javax.lang.model.element.AnnotationValue;
 import javax.lang.model.element.Element;
+import javax.lang.model.element.ElementKind;
+import javax.lang.model.element.ExecutableElement;
 import javax.lang.model.element.TypeElement;
 import javax.lang.model.element.PackageElement;
+import javax.lang.model.element.VariableElement;
 import javax.lang.model.util.*;
 import javax.tools.JavaFileManager;
 import javax.tools.StandardJavaFileManager;
 import javax.tools.JavaFileObject;
-import javax.tools.DiagnosticListener;
 
+import com.sun.source.util.AbstractTypeProcessor;
 import com.sun.source.util.TaskEvent;
 import com.sun.source.util.TaskListener;
-import com.sun.tools.javac.api.JavacTaskImpl;
+import com.sun.tools.javac.api.JavacTrees;
 import com.sun.tools.javac.code.*;
 import com.sun.tools.javac.code.Symbol.*;
-import com.sun.tools.javac.file.Paths;
+import com.sun.tools.javac.comp.Check;
 import com.sun.tools.javac.file.JavacFileManager;
 import com.sun.tools.javac.jvm.*;
 import com.sun.tools.javac.main.JavaCompiler;
+import com.sun.tools.javac.main.JavaCompiler.CompileState;
 import com.sun.tools.javac.model.JavacElements;
 import com.sun.tools.javac.model.JavacTypes;
-import com.sun.tools.javac.parser.*;
 import com.sun.tools.javac.tree.*;
 import com.sun.tools.javac.tree.JCTree.*;
 import com.sun.tools.javac.util.Abort;
+import com.sun.tools.javac.util.CancelAbort;
 import com.sun.tools.javac.util.Context;
 import com.sun.tools.javac.util.List;
 import com.sun.tools.javac.util.ListBuffer;
@@ -86,6 +95,9 @@ import static javax.tools.StandardLocation.*;
  * deletion without notice.</b>
  */
 public class JavacProcessingEnvironment implements ProcessingEnvironment, Closeable {
+
+    private static final Logger LOGGER = Logger.getLogger(JavacProcessingEnvironment.class.getName());
+
     Options options;
 
     private final boolean printProcessorInfo;
@@ -94,6 +106,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
     private final boolean lint;
     private final boolean procOnly;
     private final boolean fatalErrors;
+    private boolean foundTypeProcessors;
 
     private final JavacFiler filer;
     private final JavacMessager messager;
@@ -134,13 +147,19 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
      */
     Source source;
 
+    private ClassLoader processorClassLoader;
+
     /**
      * JavacMessages object used for localization
      */
     private JavacMessages messages;
 
+    private Check chk;
+
     private Context context;
 
+    private boolean isBackgroundCompilation;
+
     public JavacProcessingEnvironment(Context context, Iterable<? extends Processor> processors) {
         options = Options.instance(context);
         this.context = context;
@@ -154,6 +173,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
             options.get("-Xprint") != null;
         fatalErrors = options.get("fatalEnterError") != null;
         platformAnnotations = initPlatformAnnotations();
+        foundTypeProcessors = false;
 
         // Initialize services before any processors are initialzied
         // in case processors use them.
@@ -161,9 +181,11 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         messager = new JavacMessager(context, this);
         elementUtils = new JavacElements(context);
         typeUtils = new JavacTypes(context);
+        chk = Check.instance(context);
         processorOptions = initProcessorOptions(context);
         unmatchedProcessorOptions = initUnmatchedProcessorOptions();
         messages = JavacMessages.instance(context);
+        isBackgroundCompilation = options.get("backgroundCompilation") != null;     //NOI18N
         initProcessorIterator(context, processors);
     }
 
@@ -180,7 +202,6 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
     }
 
     private void initProcessorIterator(Context context, Iterable<? extends Processor> processors) {
-        Paths paths = Paths.instance(context);
         Log   log   = Log.instance(context);
         Iterator<? extends Processor> processorIterator;
 
@@ -201,7 +222,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
             JavaFileManager fileManager = context.get(JavaFileManager.class);
             try {
                 // If processorpath is not explicitly set, use the classpath.
-                ClassLoader processorCL = fileManager.hasLocation(ANNOTATION_PROCESSOR_PATH)
+                processorClassLoader = fileManager.hasLocation(ANNOTATION_PROCESSOR_PATH)
                     ? fileManager.getClassLoader(ANNOTATION_PROCESSOR_PATH)
                     : fileManager.getClassLoader(CLASS_PATH);
 
@@ -211,9 +232,9 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
                  * provider mechanism to create the processor iterator.
                  */
                 if (processorNames != null) {
-                    processorIterator = new NameProcessIterator(processorNames, processorCL, log);
+                    processorIterator = new NameProcessIterator(processorNames, processorClassLoader, log);
                 } else {
-                    processorIterator = new ServiceIterator(processorCL, log);
+                    processorIterator = new ServiceIterator(processorClassLoader, log);
                 }
             } catch (SecurityException e) {
                 /*
@@ -283,11 +304,12 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         // The to-be-wrapped iterator.
         private Iterator<?> iterator;
         private Log log;
+        private Class<?> loaderClass;
+        private boolean jusl;
+        private Object loader;
 
         ServiceIterator(ClassLoader classLoader, Log log) {
-            Class<?> loaderClass;
             String loadMethodName;
-            boolean jusl;
 
             this.log = log;
             try {
@@ -320,6 +342,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
                 // For java.util.ServiceLoader, we have to call another
                 // method to get the iterator.
                 if (jusl) {
+                    loader = result; // Store ServiceLoader to call reload later
                     Method m = loaderClass.getMethod("iterator");
                     result = m.invoke(result); // serviceLoader.iterator();
                 }
@@ -361,6 +384,18 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         public void remove() {
             throw new UnsupportedOperationException();
         }
+
+        public void close() {
+            if (jusl) {
+                try {
+                    // Call java.util.ServiceLoader.reload
+                    Method reloadMethod = loaderClass.getMethod("reload");
+                    reloadMethod.invoke(loader);
+                } catch(Exception e) {
+                    ; // Ignore problems during a call to reload.
+                }
+            }
+        }
     }
 
 
@@ -470,6 +505,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
     static class ProcessorState {
         public Processor processor;
         public boolean   contributed;
+        public boolean   invalid;
         private ArrayList<Pattern> supportedAnnotationPatterns;
         private ArrayList<String>  supportedOptionNames;
 
@@ -494,9 +530,11 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
                     if (checkOptionName(optionName, log))
                         supportedOptionNames.add(optionName);
                 }
-
+                invalid = false;
             } catch (Throwable t) {
-                throw new AnnotationProcessingError(t);
+                if (t instanceof ThreadDeath) throw (ThreadDeath) t;
+                LOGGER.log(Level.INFO, "Annotation processing error:", t);
+                invalid = true;
             }
         }
 
@@ -548,7 +586,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
      * been discoverd so far as well as the means to discover more, if
      * necessary.  A single iterator should be used per round of
      * annotation processing.  The iterator first visits already
-     * discovered processors then fails over to the service provided
+     * discovered processors then fails over to the service provider
      * mechanism if additional queries are made.
      */
     class DiscoveredProcessors implements Iterable<ProcessorState> {
@@ -560,7 +598,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
 
             ProcessorStateIterator(DiscoveredProcessors psi) {
                 this.psi = psi;
-                this.innerIter = psi.procStateList.iterator();
+                this.innerIter = new ArrayList<ProcessorState>(psi.procStateList).iterator();
                 this.onProcInterator = false;
             }
 
@@ -602,7 +640,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
                     Set<TypeElement> emptyTypeElements = Collections.emptySet();
                     while(innerIter.hasNext()) {
                         ProcessorState ps = innerIter.next();
-                        if (ps.contributed)
+                        if (!ps.invalid && ps.contributed)
                             callProcessor(ps.processor, emptyTypeElements, re);
                     }
                 }
@@ -620,12 +658,23 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
             this.processorIterator = processorIterator;
             this.procStateList = new ArrayList<ProcessorState>();
         }
+
+        /**
+         * Free jar files, etc. if using a service loader.
+         */
+        public void close() {
+            if (processorIterator != null &&
+                processorIterator instanceof ServiceIterator) {
+                ((ServiceIterator) processorIterator).close();
+            }
+        }
     }
 
     private void discoverAndRunProcs(Context context,
                                      Set<TypeElement> annotationsPresent,
                                      List<ClassSymbol> topLevelClasses,
-                                     List<PackageSymbol> packageInfoFiles) {
+                                     List<PackageSymbol> packageInfoFiles,
+                                     Map<AbstractTypeProcessor, Set<TypeElement>> typeProcessor2Types) {
         // Writer for -XprintRounds and -XprintProcessorInfo data
         PrintWriter xout = context.get(Log.outKey);
 
@@ -660,6 +709,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
 
         while(unmatchedAnnotations.size() > 0 && psi.hasNext() ) {
             ProcessorState ps = psi.next();
+            if (ps.invalid) continue;
             Set<String>  matchedNames = new HashSet<String>();
             Set<TypeElement> typeElements = new LinkedHashSet<TypeElement>();
             for (String unmatchedAnnotationName : unmatchedAnnotations.keySet()) {
@@ -672,6 +722,16 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
             }
 
             if (matchedNames.size() > 0 || ps.contributed) {
+                foundTypeProcessors = foundTypeProcessors || (ps.processor instanceof AbstractTypeProcessor);
+                if (ps.processor instanceof AbstractTypeProcessor) {
+                    Set<TypeElement> types = typeProcessor2Types.get((AbstractTypeProcessor) ps.processor);
+
+                    if (types == null) {
+                        typeProcessor2Types.put((AbstractTypeProcessor) ps.processor, types = new HashSet<TypeElement>());
+                    }
+
+                    types.addAll(ElementFilter.typesIn(renv.getRootElements()));
+                }
                 boolean processingResult = callProcessor(ps.processor, typeElements, renv);
                 ps.contributed = true;
                 ps.removeSupportedOptions(unmatchedProcessorOptions);
@@ -732,11 +792,24 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
          public Set<TypeElement> scan(Element e, Set<TypeElement> p) {
             for (AnnotationMirror annotationMirror :
                      elements.getAllAnnotationMirrors(e) ) {
-                Element e2 = annotationMirror.getAnnotationType().asElement();
-                p.add((TypeElement) e2);
+                if (isComplete(annotationMirror)) {
+                    Element e2 = annotationMirror.getAnnotationType().asElement();
+                    p.add((TypeElement) e2);
+                }
             }
             return super.scan(e, p);
         }
+
+        private boolean isComplete(AnnotationMirror annotationMirror) {
+            Map<? extends ExecutableElement, ? extends AnnotationValue> elementValues = annotationMirror.getElementValues();
+            for (Element element : annotationMirror.getAnnotationType().asElement().getEnclosedElements()) {
+                if (element.getKind() == ElementKind.METHOD) {
+                    if (!elementValues.containsKey(element) && ((ExecutableElement)element).getDefaultValue() == null)
+                        return false;
+                }
+            }
+            return true;
+        }
     }
 
     private boolean callProcessor(Processor proc,
@@ -750,7 +823,10 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
             log.error("proc.cant.access", ex.sym, ex.getDetailValue(), out.toString());
             return false;
         } catch (Throwable t) {
-            throw new AnnotationProcessingError(t);
+            if (t instanceof ThreadDeath)
+                throw (ThreadDeath)t;
+            LOGGER.log(Level.INFO, "Annotation processing error:", t);
+            return false;
         }
     }
 
@@ -768,13 +844,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         PrintWriter xout = context.get(Log.outKey);
         TaskListener taskListener = context.get(TaskListener.class);
 
-
-        AnnotationCollector collector = new AnnotationCollector();
-
         JavaCompiler compiler = JavaCompiler.instance(context);
-        compiler.todo.clear(); // free the compiler's resources
-
-        int round = 0;
 
         // List<JCAnnotation> annotationsPresentInSource = collector.findAnnotations(roots);
         List<ClassSymbol> topLevelClasses = getTopLevelClasses(roots);
@@ -796,6 +866,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
             annotationComputer.scan(classSym, annotationsPresent);
         for (PackageSymbol pkgSym : packageInfoFiles)
             annotationComputer.scan(pkgSym, annotationsPresent);
+        Map<AbstractTypeProcessor, Set<TypeElement>> typeProcessor2Types = new HashMap<AbstractTypeProcessor, Set<TypeElement>>();
 
         Context currentContext = context;
 
@@ -817,7 +888,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
                 taskListener.started(new TaskEvent(TaskEvent.Kind.ANNOTATION_PROCESSING_ROUND));
 
             try {
-                discoverAndRunProcs(currentContext, annotationsPresent, topLevelClasses, packageInfoFiles);
+                discoverAndRunProcs(currentContext, annotationsPresent, topLevelClasses, packageInfoFiles, typeProcessor2Types);
             } finally {
                 if (taskListener != null)
                     taskListener.finished(new TaskEvent(TaskEvent.Kind.ANNOTATION_PROCESSING_ROUND));
@@ -839,32 +910,22 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
                     topLevelClasses  = List.nil();
                     packageInfoFiles = List.nil();
 
-                    compiler.close(false);
-                    currentContext = contextForNextRound(currentContext, true);
-
-                    JavaFileManager fileManager = currentContext.get(JavaFileManager.class);
-
                     List<JavaFileObject> fileObjects = List.nil();
                     for (JavaFileObject jfo : filer.getGeneratedSourceFileObjects() ) {
                         fileObjects = fileObjects.prepend(jfo);
                     }
 
-
-                    compiler = JavaCompiler.instance(currentContext);
                     List<JCCompilationUnit> parsedFiles = compiler.parseFiles(fileObjects);
-                    roots = cleanTrees(roots).reverse();
-
-
-                    for (JCCompilationUnit unit : parsedFiles)
-                        roots = roots.prepend(unit);
-                    roots = roots.reverse();
-
                     // Check for errors after parsing
                     if (compiler.parseErrors()) {
                         errorStatus = true;
                         break runAround;
                     } else {
                         ListBuffer<ClassSymbol> classes = enterNewClassFiles(currentContext);
+                        roots = cleanTrees(roots).reverse();
+                        for (JCCompilationUnit unit : parsedFiles)
+                            roots = roots.prepend(unit);
+                        roots = roots.reverse();
                         compiler.enterTrees(roots);
 
                         // annotationsPresentInSource =
@@ -887,9 +948,6 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         }
         runLastRound(xout, roundNumber, errorStatus, taskListener);
 
-        compiler.close(false);
-        currentContext = contextForNextRound(currentContext, true);
-        compiler = JavaCompiler.instance(currentContext);
         filer.newRound(currentContext, true);
         filer.warnIfUnclosedFiles();
         warnIfUnmatchedOptions();
@@ -905,11 +963,10 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         * second to last round; errorRaised() gives the error status
         * of the last round.
         */
-       errorStatus = errorStatus || messager.errorRaised();
-
+        errorStatus = errorStatus || messager.errorRaised();
 
         // Free resources
-        this.close();
+        this.close(false);
 
         if (taskListener != null)
             taskListener.finished(new TaskEvent(TaskEvent.Kind.ANNOTATION_PROCESSING));
@@ -918,15 +975,22 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
             compiler.log.nerrors += messager.errorCount();
             if (compiler.errorCount() == 0)
                 compiler.log.nerrors++;
-        } else if (procOnly) {
-            compiler.todo.clear();
-        } else { // Final compilation
-            compiler.close(false);
-            currentContext = contextForNextRound(currentContext, true);
-            compiler = JavaCompiler.instance(currentContext);
+        } else if (!procOnly || foundTypeProcessors) { // Final compilation
+            updateProcessingState(currentContext, true);
+            if (procOnly && foundTypeProcessors)
+                compiler.shouldStopPolicy = CompileState.FLOW;
 
             if (true) {
-                compiler.enterTrees(cleanTrees(roots));
+                try {
+                    compiler.enterTrees(cleanTrees(roots));
+                } catch (Throwable t) {
+                    if (t instanceof ThreadDeath)
+                        throw (ThreadDeath)t;
+                    if (t instanceof CancelAbort)
+                        throw (CancelAbort)t;
+                    LOGGER.log(Level.INFO, "Error while re-entering:", t);
+                    throw new Abort(t);
+                }
             } else {
                 List<JavaFileObject> fileObjects = List.nil();
                 for (JCCompilationUnit unit : roots)
@@ -934,6 +998,8 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
                 roots = null;
                 compiler.enterTrees(compiler.parseFiles(fileObjects.reverse()));
             }
+
+            prepareAbstractTypeProcessorListener(context, typeProcessor2Types);
         }
 
         return compiler;
@@ -1012,9 +1078,20 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
     /**
      * Free resources related to annotation processing.
      */
-    public void close() {
+
+    public void close() throws IOException {
+        close(true);
+    }
+
+    public void close(boolean dropProcessors) throws IOException {
         filer.close();
-        discoveredProcs = null;
+        if (dropProcessors) {
+            if (discoveredProcs != null) // Make calling close idempotent
+                discoveredProcs.close();
+            discoveredProcs = null;
+            if (processorClassLoader != null && processorClassLoader instanceof Closeable)
+                ((Closeable) processorClassLoader).close();
+        }
     }
 
     private List<ClassSymbol> getTopLevelClasses(List<? extends JCCompilationUnit> units) {
@@ -1041,62 +1118,6 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         return packages.reverse();
     }
 
-    private Context contextForNextRound(Context context, boolean shareNames)
-        throws IOException
-    {
-        Context next = new Context();
-
-        Options options = Options.instance(context);
-        assert options != null;
-        next.put(Options.optionsKey, options);
-
-        PrintWriter out = context.get(Log.outKey);
-        assert out != null;
-        next.put(Log.outKey, out);
-
-        if (shareNames) {
-            Names names = Names.instance(context);
-            assert names != null;
-            next.put(Names.namesKey, names);
-        }
-
-        DiagnosticListener<?> dl = context.get(DiagnosticListener.class);
-        if (dl != null)
-            next.put(DiagnosticListener.class, dl);
-
-        TaskListener tl = context.get(TaskListener.class);
-        if (tl != null)
-            next.put(TaskListener.class, tl);
-
-        JavaFileManager jfm = context.get(JavaFileManager.class);
-        assert jfm != null;
-        next.put(JavaFileManager.class, jfm);
-        if (jfm instanceof JavacFileManager) {
-            ((JavacFileManager)jfm).setContext(next);
-        }
-
-        Names names = Names.instance(context);
-        assert names != null;
-        next.put(Names.namesKey, names);
-
-        Keywords keywords = Keywords.instance(context);
-        assert(keywords != null);
-        next.put(Keywords.keywordsKey, keywords);
-
-        JavaCompiler oldCompiler = JavaCompiler.instance(context);
-        JavaCompiler nextCompiler = JavaCompiler.instance(next);
-        nextCompiler.initRound(oldCompiler);
-
-        JavacTaskImpl task = context.get(JavacTaskImpl.class);
-        if (task != null) {
-            next.put(JavacTaskImpl.class, task);
-            task.updateContext(next);
-        }
-
-        context.clear();
-        return next;
-    }
-
     /*
      * Called retroactively to determine if a class loader was required,
      * after we have failed to create one.
@@ -1105,7 +1126,6 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         if (procNames != null)
             return true;
 
-        String procPath;
         URL[] urls = new URL[1];
         for(File pathElement : workingpath) {
             try {
@@ -1163,24 +1183,52 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         }
     }
 
-    private static <T extends JCTree> List<T> cleanTrees(List<T> nodes) {
+    private <T extends JCTree> List<T> cleanTrees(List<T> nodes) {
         for (T node : nodes)
             treeCleaner.scan(node);
         return nodes;
     }
 
-    private static TreeScanner treeCleaner = new TreeScanner() {
+    private TreeScanner treeCleaner = new TreeScanner() {
             public void scan(JCTree node) {
                 super.scan(node);
                 if (node != null)
                     node.type = null;
             }
             public void visitTopLevel(JCCompilationUnit node) {
-                node.packge = null;
+                if (node.packge != null) {
+                    if (node.packageAnnotations.nonEmpty())
+                        node.packge.flags_field |= Flags.APT_CLEANED;
+                    node.packge = null;
+                }
                 super.visitTopLevel(node);
             }
             public void visitClassDef(JCClassDecl node) {
-                node.sym = null;
+                if (node.sym != null) {
+                    new ElementScanner6<Void, Void>() {
+                        @Override
+                        public Void visitType(TypeElement e, Void p) {
+                            if (e instanceof ClassSymbol)
+                                ((ClassSymbol) e).flags_field |= (Flags.APT_CLEANED | Flags.FROMCLASS);
+                            return super.visitType(e, p);
+                        }
+                        @Override
+                        public Void visitExecutable(ExecutableElement e, Void p) {
+                            if (e instanceof MethodSymbol)
+                                ((MethodSymbol) e).flags_field |= (Flags.APT_CLEANED | Flags.FROMCLASS);
+                            return null;
+                        }
+                        @Override
+                        public Void visitVariable(VariableElement e, Void p) {
+                            if (e.getKind().isField() && e instanceof VarSymbol)
+                                ((VarSymbol) e).flags_field |= (Flags.APT_CLEANED | Flags.FROMCLASS);
+                            return null;
+                        }
+                    }.scan(node.sym);
+                    if (chk.compiled.get(node.sym.flatname) == node.sym)
+                        chk.compiled.remove(node.sym.flatname);
+                    node.sym = null;
+                }
                 super.visitClassDef(node);
             }
             public void visitMethodDef(JCMethodDecl node) {
@@ -1215,11 +1263,15 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
                 node.sym = null;
                 super.visitIdent(node);
             }
+            public void visitApply(JCMethodInvocation node) {
+                scan(node.typeargs);
+                super.visitApply(node);
+            }
         };
 
 
     private boolean moreToDo() {
-        return filer.newFiles();
+        return filer.newFiles() && isBackgroundCompilation;
     }
 
     /**
@@ -1397,4 +1449,110 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         }
         return true;
     }
+
+    private void prepareAbstractTypeProcessorListener(Context context, Map<AbstractTypeProcessor, Set<TypeElement>> typeProcessor2Types) {
+        if (typeProcessor2Types.isEmpty()) return;
+        
+        TaskListener otherListener = context.get(TaskListener.class);
+        if (otherListener != null) {
+            context.put(TaskListener.class, (TaskListener)null);
+        }
+        context.put(TaskListener.class, new AttributionTaskListener(context, typeProcessor2Types, otherListener));
+    }
+
+    /**
+     * A task listener that invokes the type processors whenever a class is fully
+     * analyzed.
+     */
+    private static final class AttributionTaskListener implements TaskListener {
+
+        private final Context context;
+        private final Map<AbstractTypeProcessor, Set<TypeElement>> typeProcessor2Types;
+        private final TaskListener previous;
+        private boolean hasInvokedTypeProcessingOver = false;
+
+        public AttributionTaskListener(Context context, Map<AbstractTypeProcessor, Set<TypeElement>> typeProcessor2Types, TaskListener previous) {
+            this.context = context;
+            this.typeProcessor2Types = typeProcessor2Types;
+            this.previous = previous;
+        }
+
+        @Override
+        public void finished(TaskEvent e) {
+            if (previous != null) previous.finished(e);
+
+            Log log = Log.instance(context);
+
+            maybeInvokeProcessingOver();
+
+            if (e.getKind() != TaskEvent.Kind.ANALYZE)
+                return;
+
+            if (e.getTypeElement() == null)
+                throw new AssertionError("event task without a type element");
+            if (e.getCompilationUnit() == null)
+                throw new AssertionError("even task without compilation unit");
+
+            if (log.nerrors != 0) //???seems wrong
+                return;
+            
+            TypeElement elem = e.getTypeElement();
+            TreePath p = JavacTrees.instance(context).getPath(elem);
+
+            for (Entry<AbstractTypeProcessor, Set<TypeElement>> entry : typeProcessor2Types.entrySet()) {
+                if (entry.getValue().remove(elem)) {
+                    try {
+                        entry.getKey().typeProcess(elem, p);
+                    } catch (CompletionFailure ex) {
+                        StringWriter out = new StringWriter();
+                        ex.printStackTrace(new PrintWriter(out));
+                        log.error("proc.cant.access", ex.sym, ex.getDetailValue(), out.toString());
+                        return ;
+                    } catch (Throwable t) {
+                        if (t instanceof ThreadDeath)
+                            throw (ThreadDeath)t;
+                        LOGGER.log(Level.INFO, "Annotation processing error:", t);
+                        return ;
+                    }
+                }
+            }
+
+            maybeInvokeProcessingOver();
+        }
+
+        private void maybeInvokeProcessingOver() {
+            if (hasInvokedTypeProcessingOver) return;
+
+            Log log = Log.instance(context);
+
+            if (log.nerrors != 0) return;
+            
+            for (Set<TypeElement> types : typeProcessor2Types.values()) {
+                if (!types.isEmpty()) return;
+            }
+
+            for (AbstractTypeProcessor p : typeProcessor2Types.keySet()) {
+                try {
+                    p.typeProcessingOver();
+                } catch (CompletionFailure ex) {
+                    StringWriter out = new StringWriter();
+                    ex.printStackTrace(new PrintWriter(out));
+                    log.error("proc.cant.access", ex.sym, ex.getDetailValue(), out.toString());
+                    return ;
+                } catch (Throwable t) {
+                    if (t instanceof ThreadDeath)
+                        throw (ThreadDeath)t;
+                    LOGGER.log(Level.INFO, "Annotation processing error:", t);
+                    return ;
+                }
+            }
+        }
+
+        @Override
+        public void started(TaskEvent e) {
+            if (previous != null) previous.started(e);
+        }
+
+    }
+
 }
