@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2008 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright 1999-2009 Sun Microsystems, Inc.  All Rights Reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -34,7 +34,6 @@ import java.util.Set;
 import javax.tools.DiagnosticListener;
 import javax.tools.JavaFileObject;
 
-import com.sun.tools.javac.file.JavacFileManager;
 import com.sun.tools.javac.tree.JCTree;
 import com.sun.tools.javac.api.DiagnosticFormatter;
 import com.sun.tools.javac.util.JCDiagnostic.DiagnosticPosition;
@@ -79,6 +78,12 @@ public class Log extends AbstractLog {
      */
     public boolean emitWarnings;
 
+    /** Switch: suppress note messages.
+     */
+    public boolean suppressNotes;
+
+    public boolean suppressErrorsAndWarnings;
+
     /** Print stack trace on errors?
      */
     public boolean dumpOnError;
@@ -112,6 +117,7 @@ public class Log extends AbstractLog {
 
     private final Set<Pair<JavaFileObject, Integer>> partialReparseRecorded = new HashSet<Pair<JavaFileObject,Integer>>();
     private final Map<JCTree, JCDiagnostic> errTrees = new HashMap<JCTree, JCDiagnostic>();
+    private Map<JCTree, JCDiagnostic> errTreesOverlay = null;
 
     /** Construct a log with given I/O redirections.
      */
@@ -127,6 +133,7 @@ public class Log extends AbstractLog {
         this.dumpOnError = options.get("-doe") != null;
         this.promptOnError = options.get("-prompt") != null;
         this.emitWarnings = options.get("-Xlint:none") == null;
+        this.suppressNotes = options.get("suppressNotes") != null;
         this.MaxErrors = getIntOption(options, "-Xmaxerrs", 100);
         this.MaxWarnings = getIntOption(options, "-Xmaxwarns", 100);
 
@@ -196,6 +203,23 @@ public class Log extends AbstractLog {
      *  source file name and source code position of the error is added to the set.
      */
     private Set<Pair<JavaFileObject, Integer>> recorded = new HashSet<Pair<JavaFileObject,Integer>>();
+    private Set<Pair<JavaFileObject, Integer>> recordedOverlay = null;
+
+    public void setRecordedOverlay(Set<Pair<JavaFileObject, Integer>> overlay) {
+        this.recordedOverlay = overlay;
+    }
+
+    public void pushRecordedOverlay(Set<Pair<JavaFileObject, Integer>> recordedOverlay) {
+        recorded.addAll(recordedOverlay);
+    }
+
+    public void setErrTreesOverlay(Map<JCTree, JCDiagnostic> overlay) {
+        this.errTreesOverlay = overlay;
+    }
+
+    public void pushErrTreesOverlay(Map<JCTree, JCDiagnostic> overlay) {
+        this.errTrees.putAll(overlay);
+    }
 
     public boolean hasDiagnosticListener() {
         return diagListener != null;
@@ -256,7 +280,9 @@ public class Log extends AbstractLog {
      * source name and pos.
      */
     protected boolean shouldReport(JavaFileObject file, int pos) {
-        if (multipleErrors || file == null) {
+        if (file == null) {
+            return false;
+        } else if (multipleErrors) {
             return true;
         }
         else {
@@ -269,9 +295,11 @@ public class Log extends AbstractLog {
                 return shouldReport;
             }
             else {
-                boolean shouldReport = !recorded.contains(coords);
-                if (shouldReport)
-                    recorded.add(coords);
+                boolean shouldReport = !recorded.contains(coords) && (recordedOverlay == null || !recordedOverlay.contains(coords));
+                if (shouldReport) {
+                    if (recordedOverlay != null) recordedOverlay.add(coords);
+                    else recorded.add(coords);
+                }
                 return shouldReport;
             }
         }
@@ -361,13 +389,13 @@ public class Log extends AbstractLog {
             // Print out notes only when we are permitted to report warnings
             // Notes are only generated at the end of a compilation, so should be small
             // in number.
-            if (emitWarnings || diagnostic.isMandatory()) {
+            if ((emitWarnings || diagnostic.isMandatory()) && !suppressNotes) {
                 writeDiagnostic(diagnostic);
             }
             break;
 
         case WARNING:
-            if (emitWarnings || diagnostic.isMandatory()) {
+            if ((emitWarnings || diagnostic.isMandatory()) && !suppressErrorsAndWarnings) {
                 if (nwarnings < MaxWarnings) {
                     writeDiagnostic(diagnostic);
                     nwarnings++;
@@ -376,12 +404,19 @@ public class Log extends AbstractLog {
             break;
 
         case ERROR:
-            if (diagnostic.getTree() != null && !errTrees.containsKey(diagnostic.getTree()))
-                errTrees.put(diagnostic.getTree(), diagnostic);
-            if (nerrors < MaxErrors
-                && shouldReport(diagnostic.getSource(), diagnostic.getIntPosition())) {
-                writeDiagnostic(diagnostic);
-                nerrors++;
+            if (!suppressErrorsAndWarnings) {
+                if (diagnostic.getTree() != null && !errTrees.containsKey(diagnostic.getTree())
+                        && (errTreesOverlay == null || !errTreesOverlay.containsKey(diagnostic.getTree()))) {
+                    if (errTreesOverlay != null)
+                        errTreesOverlay.put(diagnostic.getTree(), diagnostic);
+                    else
+                        errTrees.put(diagnostic.getTree(), diagnostic);
+                }
+                if (nerrors < MaxErrors
+                    && shouldReport(diagnostic.getSource(), diagnostic.getIntPosition())) {
+                    writeDiagnostic(diagnostic);
+                    nerrors++;
+                }
             }
             break;
         }
@@ -462,7 +497,7 @@ public class Log extends AbstractLog {
             JavaFileObject file = source.getFile();
             if (file != null)
                 printLines(errWriter,
-                           JavacFileManager.getJavacFileName(file) + ":" +
+                           file.getName() + ":" +
                            line + ": " + msg);
             printErrLine(pos, errWriter);
         }
